@@ -3,11 +3,6 @@
 ##
 
 plot.grain <- function(x, type, ...){
-  #cat("plot.grain; type:", type, "\n")
-   ## if (!require("Rgraphviz")){
-   ##   cat("The Rgraphviz package (from Bioconductor) must be installed to display the models\n")
-   ##   return()
-   ## }
     
    if (!requireNamespace("Rgraphviz", quietly = TRUE)) {
        cat("The Rgraphviz package (from Bioconductor) must be installed to display the models\n")
@@ -15,10 +10,10 @@ plot.grain <- function(x, type, ...){
    }
 
    if (missing(type)){
-       if (x$isCompiled){
+       if (.isComp(x)){
            Rgraphviz::plot(x$ug)
        } else {
-           if ("pot-grain" %in% class(x)){
+           if ("pot_grain" %in% class(x)){
                Rgraphviz::plot(x$ug)
            } else {
                Rgraphviz::plot(x$dag)
@@ -39,10 +34,10 @@ iplot.grain <- function(x,type, ...){
   #.primiplot(x$dag)
 
   if (missing(type)){
-    if (x$isCompiled){
+    if (.isComp(x)){
       .primiplot(x$ug)
     } else {
-      if ("pot-grain" %in% class(x)){
+      if ("pot_grain" %in% class(x)){
         .primiplot(x$ug)
       } else {
         .primiplot(x$dag)
